# Glint

- [Overview](#overview)
- [Core Features](#core-features)
  - [Entity Helpers](#entity-helpers)
  - [Preprocess Files](#preprocess-files)
- [Configuration](#configuration)
- [Advanced Features](#advanced-features)
  - [Glint Service](#glint-service)
  - [Glint Alter Hooks](#glint-alter-hooks)
- [Roadmap](#roadmap)


## Overview
Glint is a toolkit that aims to facilitate template preprocessing with entities.
The module offers a bundle of services that can be useful to your team.

## Core Features

### Entity Helpers

Entity Helpers can be used to easily access field values.

They are designed to be used at the theme layer and make accessing values much easier
than normal Drupal means.

Say you want to access the `body` field of a node. Normally you would do something like this.

```php
/**
 * Implements hook_preprocess_HOOK().
 */
function YOUR_THEME_preprocess_node(&$variables) {
  $node = $variables['node'];
  $body = $node->get('body')->getValue();
}
```

A `dump()` of `$body` would give you something akin to the following.

![Body Traditional Data](./docs/screenshots/body-field-traditional.png)

With a helper, you would do the following.

```php
/**
 * Implements hook_preprocess_HOOK().
 */
function YOUR_THEME_preprocess_node(&$variables) {
  $helper = \Drupal\glint\Glint::helper($variables['node']);
  $body = $helper->get('body');
}
```

A `dump()` of `$body` now would give you the same data, albeit slightly cleaner.

![Body Glint Data](./docs/screenshots/body-field-glint.png)

The true worth of Glint isn't for simple text fields, however. Glint shines more when trying to
access values for Drupal's trickier fields.

For example, here are examples of data results from **File Fields** and **Media References**.

```php
/**
 * Implements hook_preprocess_HOOK().
 */
function YOUR_THEME_preprocess_node(&$variables) {
  $helper = \Drupal\glint\Glint::helper($variables['node']);
  $file = $helper->get('field_document'); // File reference field.
  $media = $helper->get('field_featured_image'); // Media reference field.
}
```

![File Glint Data](./docs/screenshots/file-field-glint.png)
![Media Glint Data](./docs/screenshots/media-field-glint.png)

As you can see, the data here is much easier to work with. This helps save time and
also softens the learning curve for new devs and/or frontend experts.

### Preprocess Files

Similar to [Preprocess Functions](https://www.drupal.org/docs/8/theming-drupal-8/modifying-attributes-in-a-theme-file), 
Preprocess Files can be used to preprocess variables for any given template.

Normally, for the `node.html.twig` template, you would create a `hook_preprocess_node();` function in your theme if you
wanted to alter variables before template rendering.

With Preprocess Files, you can create a dedicated file called `node.preprocess.php` in your theme. This file will 
work exactly like a preprocess function and by default will be housed in `themes/custom/YOUR_THEME/preprocess`.

Below is an example of the contents this file will have. You can use this blueprint to create your own.

```php
<?php

/**
 * @file
 * Preprocess override for a template. Acts as a preprocess function.
 *
 * The loading of this file is provided by the Preprocess Files module.
 * @see \Drupal\glint_ppf\PreprocessFiles
 *
 * Available variables:
 * @var array $variables
 *   The variables array. The same you would get from a traditional preprocess hook.
 * @var string $hook
 *   The theme hook.
 * @var array $info
 *   The info array.
 */

// Preprocess your variables here just like you would in a hook!
$variables['foo'] = 'bar';
```

The file discovery works exactly like template file discovery, which gives you the flexibility to place your files in 
sub-folders and organize your projects how you see fit. It also works with **suggestions**, allowing you to create
`node--article.preprocess.php`, for example. 

This functionality works with **all** template hooks and suggestions, and can be used in combination with 
[Twig Debugging](https://www.drupal.org/docs/develop/theming-drupal/twig-in-drupal/debugging-twig-templates) to create preprocess files dedicated to their templates and properly segment your code and perfect 
the organization of your projects.

To use this functionality, enabled the **Preprocess Files** module, which comes bundled with Glint.

You can then configure the module at the `/admin/config/glint/preprocess-files` page.

## Configuration

All relevant configuration for the Glint module can be found at `/admin/config/glint`.

Configurations here influence the data returned when Glint performs basic data cleaning.

### Skip arrays for single value fields

Traditionally in Drupal, when called `->getValue()` on a field from an entity, you **always** get an array,
regardless of the field cardinality.

If this is something you wish to clean up when using Glint, you can activate this option.

When Glint detects that a field defined for an entity only allows 1 value (has cardinality set to 1), the singular
value is returned instead of an array containing only 1 value.

### Allow alter hooks

This functionality allows [Alter Hooks](#glint-alter-hooks) to modify the data returned by Glint.

## Advanced Features

### Glint Service

The Glint service can be called from anywhere in your code to obtain a field value from an entity.

```php
$node = Node::load(23);
$nodeBody = Glint::service()->get('body', $node);
```

This service can also be used as a shortcut to build a helper for a node.

```php
$node = Node::load(23);
$nodeHelper = Glint::helper($node);
$nodeBody = $nodeHelper->get('body');
```

If using Glint in the context of a module, it is always preferable to use [Dependency Injection](https://www.drupal.org/docs/drupal-apis/services-and-dependency-injection/services-and-dependency-injection-in-drupal-8) to access the service.
The static functions exist to facilitate access to the service from within themes.

### Glint Alter Hooks

When you obtain field values from entities using Glint, the module starts by getting the default field data. From there,
it extracts the relevant data and sometimes does additional lookups to get more relevant data that Drupal doesn't give
by default.

This process is referred to as data extraction in Glint. For advanced developers, this process can be fine-tuned with
simple hooks that allow you to further alter the data returned by Glint.

```php
/**
 * Implements hook_glint_field_extract_TYPE_alter().
 */
function YOUR_MODULE_glint_field_extract_boolean_alter(&$value, $originalValue, array $info) {
  $value = $value ? 'TRUE' : 'FALSE';
}

/**
 * Implements hook_glint_field_extract_TYPE_alter().
 */
function YOUR_THEME_glint_field_extract_text_with_summary_alter(&$value, $originalValue, array $info) {
  $value['exploded_text'] = explode(' ', $value['text']);
}
```

Using this hooks only affects the data returned when using [Helpers](#entity-helpers) or the [Glint Service](#glint-service).

## Roadmap

* Twig functions and filters. e.g. `|glint` or `|helper`, or `{{ glint() }}`
* More documentation.