<?php

namespace Drupal\glint\FieldValueCleaner;

use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\glint\Glint;
use Drupal\glint\GlintEntityHelper;
use Drupal\glint\GlintInterface;
use Drupal\media\MediaInterface;

/**
 * Provides EntityReferenceFieldValueCleaner class.
 *
 * Handles default Glint cleanup tasks for 'entity_reference' type fields.
 */
class EntityReferenceFieldValueCleaner {

  /**
   * Clean up value for an 'entity_reference' type field.
   *
   * @param array $value
   *   The original value from Drupal.
   * @param \Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem $item
   *   The item.
   * @param \Drupal\glint\GlintInterface $glint
   *   The Glint service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The Entity Repository service.
   *
   * @return \Drupal\glint\GlintEntityHelper|array
   *   A helper class to facilitate access to entity fields.
   */
  public static function clean(
    array $value,
    EntityReferenceItem $item,
    GlintInterface $glint,
    EntityRepositoryInterface $entityRepository
  ) : GlintEntityHelper | array {
    $entity = $item->entity;
    $entity = $entityRepository->getTranslationFromContext($entity);

    // For certain entity types, Glint will return a simple array instead of a
    // helper.
    return match ($entity->getEntityTypeId()) {
      'media' => self::cleanMediaEntity($value, $item, $glint, $entity),
      default => new GlintEntityHelper($glint, $entity),
    };
  }

  /**
   * Perform cleaning for Media entities.
   *
   * These entities are generally simple and have common data.
   *
   * @param array $value
   *   The original value from Drupal.
   * @param \Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem $item
   *   The item.
   * @param \Drupal\glint\GlintInterface $glint
   *   The Glint service.
   * @param \Drupal\media\MediaInterface $entity
   *   The media entity.
   *
   * @return array
   *   Array of relevant data from the media.
   */
  private static function cleanMediaEntity(
    array $value,
    EntityReferenceItem $item,
    GlintInterface $glint,
    MediaInterface $entity
  ) : array {

    // Get a glint output of all fields in the media.
    $helper = new GlintEntityHelper($glint, $entity);
    $values = $helper->all();

    // Include the media title.
    $values['media_title'] = $entity->label();

    // Merge key.
    // Depending on the media type, we merge the value of a field into the first
    // level of the array.
    // Ease of access when preprocessing!
    $mergeKey = match($entity->getSource()->getPluginId()) {
      'image' => 'field_media_image',
      'video_file' => 'field_media_video_file',
      'oembed:video' => 'field_media_oembed_video',
      'file' => 'field_media_file',
    };

    // Merge.
    if (isset($values[$mergeKey]) && is_array($values[$mergeKey])) {
      $data = $values[$mergeKey];
      // If we don't skip arrays well we skip them here.
      if (!$glint->config()->get(Glint::CONFIG__SKIP_ARRAYS)) {
        $data = $data[0];
      }
      unset($values[$mergeKey]);
      $values = array_merge($data, $values);
    }

    // Additional actions for remote videos.
    if ($entity->getSource()->getPluginId() === 'oembed:video') {
      $youtubeRegex = '/^(https?:\/\/)?(www\.youtube\.com|youtu\.?be)\/.+$/';
      $vimeoRegex = '/vimeo\.com\/\d+/';
      $matches = [];
      $idRegex = NULL;

      if (preg_match($youtubeRegex, $values['field_media_oembed_video'])) {
        $values['provider'] = 'youtube';
        $idRegex = '/^.*(?:youtu\.be\/|v\/|u\/\w\/|embed\/|watch\?v=|&v=)([^#&?]*).*$/i';
      }

      if (preg_match($vimeoRegex, $values['field_media_oembed_video'])) {
        $values['provider'] = 'vimeo';
        $idRegex = '/(?:www\.|player\.)?vimeo.com\/(?:channels\/(?:\w+\/)?|groups\/[^\/]*\/videos\/|album\/\d+\/video\/|video\/|)(\d+)(?:[a-zA-Z\d_\-]+)?/i';
      }

      if ($idRegex !== NULL) {
        preg_match($idRegex, $values['field_media_oembed_video'], $matches);
        $values['video_id'] = $matches[1] ?? NULL;
      }

      if (isset($values['field_media_oembed_video'])) {
        $url = $values['field_media_oembed_video'];
        unset($values['field_media_oembed_video']);
        $values['url'] = $url;
      }
    }

    return $values;
  }

}
